@@ -5,22 +5,24 @@ from PySide6.QtWidgets import QMenu
 def insert(self, el, anchor=None):
     if isinstance(el, QMenu):
         if anchor:
-            self.insertMenu(anchor, el)
+            action = self.insertMenu(anchor, el)
         else:
-            self.addMenu(el)
+            action = self.addMenu(el)
+        action.setParent(self.menuAction())
     elif isinstance(el, QAction):
         if anchor:
             self.insertAction(anchor, el)
         else:
             self.addAction(el)
-    el.setParent(self)
+        el.setParent(self.menuAction())
 
 
 def remove(self, el):
     if isinstance(el, QAction):
         self.removeAction(el)
+        el.setParent(None)
     elif isinstance(el, QMenu):
         el.clear()
         menu_action = el.menuAction()
         self.removeAction(menu_action)
-    el.setParent(None)
+        menu_action.setParent(None)
